// 代码行数
const fs = require("fs");
const babel = require("@babel/core");
/**
 *
 * @param {*} path 当前节点路径
 */
const funcDetail = ({ filePath, path, prefixName }) => {
  const { loc, params } = path.node;
  const start = loc.start.line;
  const end = loc.end.line;
  
  return {
    funcLoc: `${getFilePath(filePath)}[${start}:${end}]`,
    funcName: prefixName ? prefixName.name : "-",
    funcParams: params.length,
    funcType: path.node.type,
    funcLine: end - start + 1,
  };
};

// 获取文件位置
const getFilePath = (filePath) => {
  return filePath.replace(process.cwd(), "").trim();
};

const getLine = (filePath) => {
  try {
    const fileContent = fs.readFileSync(filePath, "utf-8");
    let resultLine = {
      importLine: 0,
    };
    const func = [];

    const visitor = {
      ImportDeclaration() {
        resultLine.importLine++;
      },
      Import() {
        resultLine.importLine++;
      },
      CallExpression(path) {
        const callee = path.get("callee");
        if (
          callee.isIdentifier({ name: "require" }) &&
          path.get("arguments")[0].isStringLiteral()
        ) {
          resultLine.importLine++;
        }
      },
      ArrowFunctionExpression(path) {
        const parentPath = path.parentPath;
        func.push(
          funcDetail({
            filePath,
            path,
            prefixName: parentPath.node.id,
          })
        );
      },
      FunctionDeclaration(path) {
        func.push(
          funcDetail({
            filePath,
            path,
            prefixName: path.node.id,
          })
        );
      },
      FunctionExpression(path) {
        func.push(
          funcDetail({
            filePath,
            path,
            prefixName: path.node.id,
          })
        );
      },
      ClassMethod(path) {
        func.push(
          funcDetail({
            filePath,
            path,
            prefixName: path.node.key,
          })
        );
      },
      ObjectMethod(path) {
        func.push(
          funcDetail({
            filePath,
            path,
            prefixName: path.node.key,
          })
        );
      },
    };

    const ast = require("@babel/parser").parse(fileContent, {
      sourceType: "module",
      plugins: ["jsx", "typescript"],
    });
    babel.traverse(ast, visitor);
    const { loc, comments } = ast;
    return {
      ...resultLine,
      commentLine: comments.length,
      allLine: loc.end.line,
      func,
      filePath: getFilePath(filePath),
    };
  } catch (error) {
    console.error("line", error);
    process.exit(1);
  }
};

const getCodeLine = (files) => {
  return files.map((file) => getLine(file)).flat();
};

/**
 *
 * @param {*} files scan 扫描出来的文件
 * filename 文件名称
 * allLine 总行数
 * commentLine 注释行数
 * blankLine 空行
 * codeLine 代码行数
 */
module.exports = function codeLine(files) {
  return getCodeLine(files);
};
